using GData;

namespace GWoffice.Services
{
	
	public class Sync
	{
		
		DocumentsService docs;
		App app;
		
		HashTable<string,string> etags;
		
		public Sync (DocumentsService _docs, App _app)
		{
			docs = _docs;
			app = _app;
		}
		
		public async void resync ()
		{
			done = 0;
			
			app.progress_item.show ();
			app.progress_label.label = _("Resyncing...");
			app.progress_bar.fraction = 0;
			
			//get stored etags into a hashtable for easy access
			etags = new HashTable<string,string> (str_hash, str_equal);
			string etags_data = "";
			try {
				FileUtils.get_contents (app.etags_file.get_path (), out etags_data);
				
				var splitted = etags_data.split ("\n");
				for (var i=0;i<splitted.length;i+=2)
					etags.set (splitted[i], splitted[i+1]); //path, etag
				
			} catch (Error e) { warning (e.message); }
			
			app.launcher.progress = 0;
			app.launcher.count = app.total_count;
			app.launcher.progress_visible = true;
			app.launcher.count_visible = true;
			
			if (!app.sync_dir.query_exists ()) {
				try {
					app.sync_dir.make_directory ();
				} catch (Error e) { warning (e.message); }
			}
			
			etags_content = "";
			
			Thread.create<void*> (() => {
				list_drive_rec ("https://docs.google.com/feeds/default/private/full/folder%3Aroot", 
					app.sync_dir, 0);
				
				try {
					FileUtils.set_contents (app.etags_file.get_path (), etags_content);
				} catch (Error e) { warning (e.message); }
				
				app.launcher.progress_visible = false;
				app.launcher.count_visible = false;
				app.progress_item.hide ();
				
				return null;
			}, false);
			
		}
		
		//count of elements that have been processed
		int done;
		//new content for etags file
		string etags_content;
		
		
		signal void progress (double val);
		
		/*recursively go through the directory structure*/
		public void list_drive_rec (string id, File current_file, int level)
		{
			var q = new DocumentsQuery (null);
			q.show_folders = true;
			q.folder_id = id;
			try {
				
				var feed = docs.query_documents (q, null, null);
				
				for (var i=0;i<feed.get_entries ().length ();i++) {
					var entry = feed.get_entries ().nth_data (i);
					
					if (Utils.entry_is_folder (entry)) {
						
						var folder = File.new_for_path (current_file.get_path ()+"/"+entry.title);
						if (!folder.query_exists ())
							folder.make_directory ();
						
						list_drive_rec (entry.id, folder, level+1);
					} else {
						
						done ++;
						
						var type = GData.DocumentsTextFormat.PDF;
						switch ((entry as GData.DocumentsEntry).resource_id.split (":")[0].replace (":", "")) {
							case "spreadsheet":
								type = Services.Settings.get_default ().spreadsheet_format;
								break;
							case "presentation":
								type = Services.Settings.get_default ().presentation_format;
								break;
							case "document":
								type = Services.Settings.get_default ().text_format;
								break;
						}
						
						var path = File.new_for_path (current_file.get_path ()+"/"+entry.title+"."+type);
						var etag = etags.lookup (path.get_path ());
						//there's a more recent version, download
						if (etag != entry.etag) {
							debug ("Need to download: %s (%s!=%s)\n", entry.title, etag, entry.etag);
							
							try {
								if (path.query_exists ())
									path.delete ();
								FileOutputStream out_stream = path.create (FileCreateFlags.REPLACE_DESTINATION, null);
								
								var in_stream = (entry as DocumentsDocument).download (docs, type, null);
								out_stream.splice (in_stream, OutputStreamSpliceFlags.CLOSE_SOURCE | 
									OutputStreamSpliceFlags.CLOSE_TARGET);
							} catch (Error e) { warning ("Could not download '%s', %s", entry.title, e.message); }
						}
						etags_content += path.get_path ()+"\n"+entry.etag+"\n";
						
						app.launcher.progress = ((double)done / app.total_count);
						app.progress_bar.fraction = app.launcher.progress;
					}
				}
			
			} catch (Error e) { warning (e.message); }
		}
	}
}

