using GData;

namespace GWoffice.Widgets
{
	
	public class FileView : Gtk.TreeView
	{
		
		App app;
		
		public signal void selected ();
		
		internal Gtk.TreeModelFilter filter;
		public Gtk.ListStore file_model;
		
		string _filter_string = "";
		public string filter_string {
			get { return _filter_string; }
			set {
				_filter_string = value;
				filter.refilter ();
			}
		}
		
		bool _only_mine;
		public bool only_mine {
			get { return _only_mine; }
			set {
				_only_mine = value;
				filter.refilter ();
			}
		}
		
		public FileView (App _app, Gtk.ListStore? _model=null)
		{
			app = _app;
			
			if (_model == null)
				file_model = new Gtk.ListStore (6, typeof (Gdk.Pixbuf),  //mime icon
											   typeof (string),    //filename
											   typeof (string),    //date
											   typeof (DocumentsDocument), //document
											   typeof (string),   //icon name
											   typeof (bool));    //if it's my doc
			else
				file_model = _model;
			
			filter = new Gtk.TreeModelFilter (file_model, null);
			model = filter;
			
			filter.set_visible_func ((model, it) => {
				string name;
				bool mine;
				model.get (it, 1, out name, 5, out mine);
				if (name == null)
					return false;
				return (name.down ().index_of (filter_string.down ()) != -1) && (mine || !only_mine);
			});
			
			var title_render = new Gtk.CellRendererText ();
			title_render.ellipsize = Pango.EllipsizeMode.END;
			title_render.width = 330;
			var date_render = new Gtk.CellRendererText ();
			date_render.width = 80;
			
			insert_column_with_attributes (-1, null, new Gtk.CellRendererPixbuf (), "pixbuf", 0);
			insert_column_with_attributes (-1, null, title_render, "text", 1);
			insert_column_with_attributes (-1, null, date_render, "text", 2);
			
			headers_visible = false;
			
			var css = new Gtk.CssProvider ();
			try {
				css.load_from_data ("
					* {
						background-color: @background-color;
					}
					.view:selected:focused {
						color: @selected_fg_color;
						background-color: @selected_bg_color;
					}
					", -1);
			} catch (Error e) { warning (e.message); }
			get_style_context ().add_provider (css, 20000);
		}
		
		public override void row_activated (Gtk.TreePath path, Gtk.TreeViewColumn col)
		{
			Gtk.TreeIter iter;
			DocumentsDocument doc;
			string title, icon_name;
			
			filter.get_iter (out iter, path);
			filter.get (iter, 1, out title, 3, out doc, 4, out icon_name);
			
			app.open_document (doc, icon_name);
			
			selected ();
		}
	
		public void add_item (DocumentsEntry doc, bool mine)
		{
			string icon;
			
			var type = doc.resource_id.split (":")[0].replace (":", "");
			switch (type) {
				case "spreadsheet":
				case "presentation":
				case "document":
					icon = "x-office-"+type;
					break;
				default:
					icon = "text-x-generic-template";
					break;
			}
			
			try {
				Gtk.TreeIter iter;
				(file_model as Gtk.ListStore).append (out iter);
				(file_model as Gtk.ListStore).set (iter, 
					0, Gtk.IconTheme.get_default ().load_icon (icon, 32, 0), 
					1, doc.title, 
					2, new DateTime.from_unix_local (doc.edited).format ("%x"), 
					3, doc,
					4, icon,
					5, mine);
			} catch (Error e) { warning (e.message); }
		}
	}
}

