using GData;


namespace GWoffice.Widgets
{
	
	public class Preferences : Gtk.Dialog
	{
		
		
		public Preferences (App app)
		{
			
			resizable = false;
			width_request = 300;
			
			title = _("Preferences");
			
			var box = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
			box.margin = 12;
			var close = new Gtk.Button.with_label (_("Close"));
			close.halign = Gtk.Align.END;
			
#if HAS_NEW_GRANITE
		var notebook = new Granite.Widgets.StaticNotebook (true);
#else
		var notebook = new Granite.Widgets.StaticNotebook ();
#endif
			
			/*general*/
			var grid_gen = new Gtk.Grid ();
			grid_gen.column_spacing = 6;
			grid_gen.row_spacing = 6;
			grid_gen.column_homogeneous = true;
			
			var style = new Gtk.CheckButton ();
			style.active = Services.Settings.get_default ().style;
			style.toggled.connect (() => Services.Settings.get_default ().style = style.active );
			
			var compact_view = new Gtk.CheckButton ();
			compact_view.active = Services.Settings.get_default ().always_show_compact_view;
			compact_view.toggled.connect (() => Services.Settings.get_default ().always_show_compact_view = compact_view.active );
			
			grid_gen.attach (new LLabel.markup ("<b>"+_("Appearance:")+"</b>"), 0, 0, 1, 1);
			grid_gen.attach (new LLabel.right (_("Apply Theme to Editor:")), 0, 1, 1, 1);
			grid_gen.attach (style, 1, 1, 1, 1);
			grid_gen.attach (new LLabel (_("Note: Theme is experimental and might cause GWoffice to crash")), 0, 2, 2, 1);
			grid_gen.attach (new LLabel.right (_("Always use Compact Controls:")), 0, 3, 1, 1);
			grid_gen.attach (compact_view, 1, 3, 1, 1);
			
			/*formats*/
			var grid_formats = new Gtk.Grid ();
			grid_formats.column_spacing = 6;
			grid_formats.row_spacing = 6;
			grid_formats.column_homogeneous = true;
			
			var text_format = new Gtk.ComboBoxText ();
			text_format.append (DocumentsTextFormat.DOC, DocumentsTextFormat.DOC);
			text_format.append (DocumentsTextFormat.HTML, DocumentsTextFormat.HTML);
			text_format.append (DocumentsTextFormat.ODT, DocumentsTextFormat.ODT);
			text_format.append (DocumentsTextFormat.PDF, DocumentsTextFormat.PDF);
			text_format.append (DocumentsTextFormat.PNG, DocumentsTextFormat.PNG);
			text_format.append (DocumentsTextFormat.RTF, DocumentsTextFormat.RTF);
			text_format.append (DocumentsTextFormat.TXT, DocumentsTextFormat.TXT);
			text_format.append (DocumentsTextFormat.ZIP, DocumentsTextFormat.ZIP);
			text_format.active_id = Services.Settings.get_default ().text_format;
			text_format.changed.connect (() => {
				Services.Settings.get_default ().text_format = text_format.active_id;
				app.clear_sync_folder ();
			});
			
			var presentation_format = new Gtk.ComboBoxText ();
			presentation_format.append (DocumentsPresentationFormat.PDF, DocumentsPresentationFormat.PDF);
			presentation_format.append (DocumentsPresentationFormat.PNG, DocumentsPresentationFormat.PNG);
			presentation_format.append (DocumentsPresentationFormat.PPT, DocumentsPresentationFormat.PPT);
			presentation_format.append (DocumentsPresentationFormat.SWF, DocumentsPresentationFormat.SWF);
			presentation_format.append (DocumentsPresentationFormat.TXT, DocumentsPresentationFormat.TXT);
			presentation_format.active_id = Services.Settings.get_default ().presentation_format;
			presentation_format.changed.connect (() => {
				Services.Settings.get_default ().presentation_format = presentation_format.active_id;
				app.clear_sync_folder ();
			});
			
			var spreadsheet_format = new Gtk.ComboBoxText ();
			spreadsheet_format.append (DocumentsSpreadsheetFormat.CSV, DocumentsSpreadsheetFormat.CSV);
			spreadsheet_format.append (DocumentsSpreadsheetFormat.HTML, DocumentsSpreadsheetFormat.HTML);
			spreadsheet_format.append (DocumentsSpreadsheetFormat.ODS, DocumentsSpreadsheetFormat.ODS);
			spreadsheet_format.append (DocumentsSpreadsheetFormat.PDF, DocumentsSpreadsheetFormat.PDF);
			spreadsheet_format.append (DocumentsSpreadsheetFormat.TSV, DocumentsSpreadsheetFormat.TSV);
			spreadsheet_format.append (DocumentsSpreadsheetFormat.XLS, DocumentsSpreadsheetFormat.XLS);
			spreadsheet_format.active_id = Services.Settings.get_default ().spreadsheet_format;
			spreadsheet_format.changed.connect (() => {
				Services.Settings.get_default ().spreadsheet_format = spreadsheet_format.active_id;
				app.clear_sync_folder ();
			});
			
			grid_formats.attach (new LLabel.markup ("<b>"+_("Preferred Formats:")+"</b>"), 0, 0, 2, 1);
			grid_formats.attach (new LLabel.right (_("Texts:")), 0, 1, 1, 1);
			grid_formats.attach (text_format, 1, 1, 1, 1);
			grid_formats.attach (new LLabel.right (_("Presentations:")), 0, 2, 1, 1);
			grid_formats.attach (presentation_format, 1, 2, 1, 1);
			grid_formats.attach (new LLabel.right (_("Spreadsheets:")), 0, 3, 1, 1);
			grid_formats.attach (spreadsheet_format, 1, 3, 1, 1);
			grid_formats.attach (new LLabel.wrap (_("Note: changes will delete the sync folder and resync after restart.")), 0, 4, 2, 1);
			
			/*sync*/
			var grid_sync = new Gtk.Grid ();
			grid_sync.column_spacing = 6;
			grid_sync.row_spacing = 6;
			grid_sync.column_homogeneous = true;
			
			var open_uploaded = new Gtk.CheckButton ();
			open_uploaded.active = Services.Settings.get_default ().open_uploaded;
			open_uploaded.toggled.connect (() => Services.Settings.get_default ().open_uploaded = open_uploaded.active );
			var resync_on_startup = new Gtk.CheckButton ();
			resync_on_startup.active = Services.Settings.get_default ().resync_on_startup;
			resync_on_startup.toggled.connect (() => Services.Settings.get_default ().resync_on_startup = resync_on_startup.active );
			var download_docs = new Gtk.CheckButton ();
			download_docs.active = Services.Settings.get_default ().download_docs;
			download_docs.toggled.connect (() => {
				Services.Settings.get_default ().download_docs = download_docs.active;
				resync_on_startup.sensitive = download_docs.active;
			});
			resync_on_startup.sensitive = download_docs.active;
			
			grid_sync.attach (new LLabel.markup ("<b>"+_("Up:")+"</b>"), 0, 0, 1, 1);
			grid_sync.attach (new LLabel.right (_("Open uploaded documents:")), 0, 1, 1, 1);
			grid_sync.attach (open_uploaded, 1, 1, 1, 1);
			
			grid_sync.attach (new LLabel.markup ("<b>"+_("Down:")+"</b>"), 0, 2, 1, 1);
			grid_sync.attach (new LLabel.right (_("Download all documents:")), 0, 3, 1, 1);
			grid_sync.attach (download_docs, 1, 3, 1, 1);
			grid_sync.attach (new LLabel.right (_("Resync on startup:")), 0, 4, 1, 1);
			grid_sync.attach (resync_on_startup, 1, 4, 1, 1);
			grid_sync.attach (new LLabel (_("Note: Resyncing will overwrite any changes you did locally.")), 0, 5, 2, 1);
			
			notebook.append_page (grid_gen, new Gtk.Label (_("General")));
			notebook.append_page (grid_formats, new Gtk.Label (_("Formats")));
			notebook.append_page (grid_sync, new Gtk.Label (_("Syncing")));
			
			box.pack_start (notebook);
			box.pack_start (close);
			
			close.clicked.connect (() => destroy ());
			
			(get_content_area () as Gtk.Container).add (box);
			get_content_area ().show_all ();
		}
		
		
	}
	
}

